# -*- coding: utf-8 -*-
import time


# 退出当前应用程序
def exitApplication(cls, exitValue):
    if cls.commandScript is not None:
        # 运行在脚本模式下，会一直等待所有子进程退出后，再退出本程序
        while True:
            if not cls.JobHandler.isAllJobClosed():
                # 如果还有没有退出的进程，则不会直接退出程序，会继续等待进程退出
                time.sleep(3)
            else:
                break

        # 等待那些已经Running的进程完成， 但是只有Submitted的不考虑在内
        if cls.testOptions.get("JOBMANAGER") == "TRUE":
            cls.JobHandler.waitjob("all")

        # 断开数据库连接
        if cls.db_conn:
            cls.db_conn.close()
        cls.db_conn = None
        cls.cmdExecuteHandler.sqlConn = None

        # 断开之前保存的其他数据库连接
        for m_conn in cls.db_saved_conn.values():
            m_conn[0].close()

        # 退出应用程序
        cls.exitValue = exitValue
        raise EOFError
    else:
        if cls.testOptions.get("JOBMANAGER") == "ON":
            # 运行在控制台模式下
            if not cls.JobHandler.isAllJobClosed():
                yield {
                    "title": None,
                    "rows": None,
                    "headers": None,
                    "columnTypes": None,
                    "status": "Please wait all background process complete."
                }
            else:
                # 退出应用程序
                cls.exitValue = exitValue
                raise EOFError
        else:
            # 退出应用程序
            cls.exitValue = exitValue
            raise EOFError



# -*- coding: utf-8 -*-
from ..testcliexception import TestCliException


# 数据库会话管理
def sessionManage(cls, action: str, sessionName: str = None):
    # Session_Context:
    #   0:   Connection
    #   1:   UserName
    #   2:   Password
    #   3:   URL
    if action.strip().lower() == 'show':
        result = []
        for m_Session_Name, m_Connection in cls.db_saved_conn.items():
            if m_Connection[0] is None:
                result.append(['None', str(m_Session_Name), str(m_Connection[1]), '******', str(m_Connection[3])])
            else:
                result.append(['Connection', str(m_Session_Name), str(m_Connection[1]), '******', str(m_Connection[3])])
        if cls.db_conn is not None:
            result.append(['Current', str(cls.db_sessionName), str(cls.db_username), '******', str(cls.db_url)])
        if len(result) == 0:
            yield {
                "type": "result",
                "title": None,
                "rows": None,
                "headers": None,
                "columnTypes": None,
                "status": "No saved sesssions."
            }
        else:
            yield {
                "type": "result",
                "title": "Saved Sessions:",
                "rows": result,
                "headers": ["Session", "Sesssion Name", "User Name", "Password", "URL"],
                "columnTypes": None,
                "status": "Total " + str(len(result)) + " saved sesssions."
            }
        return

    if action.strip().lower() == 'release':
        if cls.db_conn is None:
            raise TestCliException(
                "You don't have a saved session.")
        del cls.db_saved_conn[sessionName]
        cls.db_sessionName = None
    elif action.strip().lower() == 'save':
        if cls.db_conn is None:
            raise TestCliException(
                "Please connect session first before save.")
        cls.db_saved_conn[sessionName] = [cls.db_conn, cls.db_username, cls.db_password, cls.db_url]
        cls.db_sessionName = sessionName
    elif action.strip().lower() == 'saveurl':
        if cls.db_conn is None:
            raise TestCliException(
                "Please connect session first before save.")
        cls.db_saved_conn[sessionName] = [None, cls.db_username, cls.db_password, cls.db_url]
        cls.db_sessionName = sessionName
    elif action.strip().lower() == 'restore':
        if sessionName in cls.db_saved_conn:
            cls.db_username = cls.db_saved_conn[sessionName][1]
            cls.db_password = cls.db_saved_conn[sessionName][2]
            cls.db_url = cls.db_saved_conn[sessionName][3]
            if cls.db_saved_conn[sessionName][0] is None:
                result = cls.connect_db(cls.db_username + "/" + cls.db_password + "@" + cls.db_url)
                for title, cur, headers, columnTypes, status in result:
                    yield {
                        "title": title,
                        "rows": cur,
                        "headers": headers,
                        "columnTypes": columnTypes,
                        "status": status
                    }
            else:
                cls.db_conn = cls.db_saved_conn[sessionName][0]
                cls.cmdExecuteHandler.sqlConn = cls.db_conn
                cls.db_sessionName = sessionName
        else:
            raise TestCliException(
                "Session [" + sessionName + "] does not exist. Please save it first.")
    else:
        raise TestCliException(
            "Wrong argument : " + "Session save/restore [session name]")
    if action.strip().lower() == 'save':
        yield {
            "type": "result",
            "title": None,
            "rows": None,
            "headers": None,
            "columnTypes": None,
            "status": "Session saved Successful."
        }
    if action.strip().lower() == 'release':
        yield {
            "type": "result",
            "title": None,
            "rows": None,
            "headers": None,
            "columnTypes": None,
            "status": "Session release Successful."
        }
    if action.strip().lower() == 'restore':
        cls.testOptions.set("CONNURL", cls.db_url)
        yield {
            "type": "result",
            "title": None,
            "rows": None,
            "headers": None,
            "columnTypes": None,
            "status": "Session restored Successful."
        }



# -*- coding: utf-8 -*-

import ast
import re
from io import open
from setuptools import setup

'''
How to build and upload this package to PyPi
    python setup.py sdist
    python setup.py bdist_wheel --universal
    twine upload dist/*

How to build and upload this package to Local site:
    python setup.py install
'''


_version_re = re.compile(r"__version__\s+=\s+(.*)")

with open("testcli/__init__.py", "rb") as f:
    version = str(
        ast.literal_eval(_version_re.search(f.read().decode("utf-8")).group(1))
    )


def open_file(filename):
    """Open and read the file *filename*."""
    with open(filename, 'r+', encoding='utf-8') as rf:
        return rf.read()


readme = open_file("testcli/docs/UserGuide.md")

setup(
    name='robotslacker-testcli',
    version=version,
    description='Test Command tool',
    long_description=readme,
    keywords='test command sql api',
    long_description_content_type='text/markdown',
    platforms='any',
    install_requires=['JPype1', 'setproctitle', 'urllib3<1.27',
                      'click', 'prompt_toolkit',  'paramiko', 'antlr4-python3-runtime',
                      'hdfs', 'fs', "psutil",
                      "rich",
                      "pytest", 'fastapi', 'uvicorn',
                      "coloredlogs", "robotframework", "beautifulsoup4", "lxml"
                      ],

    author='RobotSlacker',
    author_email='184902652@qq.com',
    url='https://github.com/robotslacker/testcli.git',

    zip_safe=False,
    packages=['testcli'],
    package_data={'testcli': [
        'jlib/*',
        'conf/*', 'profile/*',
        'antlrgen/*', 'commands/*',
        'test/*',
        'docs/UserGuide.md'
    ]},
    python_requires='>=3.6',
    entry_points={
        "console_scripts": ["testcli = testcli.main:cli"],
        "distutils.commands": ["lint = tasks:lint", "test = tasks:test"],
    },
)



# -*- coding: utf-8 -*-
import os
from ..testcliexception import TestCliException


# 设置一些选项
def setOptions(cls, options):
    if "optionName" in options:
        optionName = options["optionName"]
    else:
        optionName = None
    if "optionValue" in options:
        optionValue = options["optionValue"]
    else:
        optionValue = ""

    if optionName is None:
        # SET如果没有参数，则显示所有的选项出来
        result = []
        for row in cls.testOptions.getOptionList():
            if not row["Hidden"]:
                result.append([row["Name"], row["Value"], row["Comments"]])
        yield {
            "type": "result",
            "title": "Current Options: ",
            "rows": result,
            "headers": ["Name", "Value", "Comments"],
            "columnTypes": None,
            "status": None
        }
        return

    if optionName.upper() == "JOBMANAGER":
        # JOBMANAGER无法通过SET命令来设置
        yield {
            "type": "error",
            "message": "Can't update this option with set command. Please use '_JOB JOBMANAGER ON|OFF'."
        }
        return
    if optionName.upper() == "MONITORMANAGER":
        # MONITORMANAGER无法通过SET命令来设置
        yield {
            "type": "error",
            "message": "Can't update this option with set command. Please use '_MONITOR MONITORMANAGER ON|OFF'."
        }
        return
    if optionName.upper() == "WHENEVER_ERROR":
        # WHENEVER无法通过SET命令来设置
        yield {
            "type": "error",
            "message": "Can't update this option with set command. Please use '_WHENEVER ERROR <EXIT <int> | CONTINUE>'."
        }
        return

    # 以下参数只能为ON或者OFF
    if optionName.upper() in ["DEBUG", "TIMING", "TIME", "ECHO", "PAGE", "TERMOUT", "FEEDBACK",
                              "OUTPUT_SORT_ARRAY", "OUTPUT_CSV_HEADER", "SILENT"]:
        if optionValue.upper() not in ['ON', 'OFF']:
            yield {
                "type": "error",
                "message": "Option is ON/OFF only'."
            }
            return

    # 以下参数只能为整形
    if optionName.upper() in ["SQL_FETCHSIZE", "LOB_LENGTH", "SQLCONN_RETRYTIMES"]:
        try:
            optionValue = int(str(optionValue))
            if optionValue <=0:
                yield {
                    "type": "error",
                    "message": "Option is valid positive integer only."
                }
                return
        except ValueError:
            yield {
                "type": "error",
                "message": "Option is valid integer only."
            }
            return
    if optionName.upper() in ["SCRIPT_TIMEOUT", "SQL_TIMEOUT", "API_TIMEOUT"]:
        try:
            optionValue = int(str(optionValue))
            if optionValue <=0 and optionValue != -1:
                yield {
                    "type": "error",
                    "message": "Option is valid positive integer(or -1 means unlimited) only."
                }
                return
        except ValueError:
            yield {
                "type": "error",
                "message": "Option is valid integer only."
            }
            return

    # 处理DEBUG选项
    if optionName.upper() == "DEBUG":
        if optionValue.upper() == 'ON':
            os.environ['TESTCLI_DEBUG'] = "1"
        else:
            if 'TESTCLI_DEBUG' in os.environ:
                del os.environ['TESTCLI_DEBUG']
        yield {
            "type": "result",
            "title": None,
            "rows": None,
            "headers": None,
            "columnTypes": None,
            "status": None
        }
        return

    # 处理AUTOCOMMIT选项
    if optionName.upper() == "AUTOCOMMIT":
        if cls.db_conn is None:
            raise TestCliException("Not connected.")
        if optionValue.upper() == 'FALSE':
            cls.db_conn.setAutoCommit(False)
        elif optionValue.upper() == 'TRUE':
            cls.db_conn.setAutoCommit(True)
        else:
            raise TestCliException("Unknown option [" + str(optionValue) + "]. True/False only.")
        yield {
            "type": "result",
            "title": None,
            "rows": None,
            "headers": None,
            "columnTypes": None,
            "status": None
        }
        return

    # 对于子进程，连接到JOB管理服务
    if optionName.upper() == "JOBMANAGER_METAURL":
        if cls.testOptions.get("JOBMANAGER") == "ON":
            raise TestCliException("You can't act as worker rule while option JOBMANAGER is ON.")
        jobManagerURL = optionValue
        if len(jobManagerURL) == 0:
            # 退出Meta的连接
            cls.MetaHandler.DisConnectServer()
            cls.JobHandler.setMetaConn(None)
            cls.TransactionHandler.setMetaConn(None)
            cls.testOptions.set("JOBMANAGER_METAURL", "")
        else:
            # 对于被主进程调用的进程，则不需要考虑, 连接到主进程的Meta服务
            cls.MetaHandler.ConnectServer(jobManagerURL)
            cls.JobHandler.setMetaConn(cls.MetaHandler.db_conn)
            cls.TransactionHandler.setMetaConn(cls.MetaHandler.db_conn)
            cls.testOptions.set("JOBMANAGER_METAURL", jobManagerURL)
        yield {
            "type": "result",
            "title": None,
            "rows": None,
            "headers": None,
            "columnTypes": None,
            "status": None
        }
        return

    # 查看是否属于定义的选项
    if cls.testOptions.get(optionName.upper()) is not None:
        cls.testOptions.set(optionName.upper(), optionValue)
        yield {
            "type": "result",
            "title": None,
            "rows": None,
            "headers": None,
            "columnTypes": None,
            "status": None
        }
        return
    else:
        # 不认识的配置选项
        yield {
            "type": "error",
            "message": "Unknown option [" + str(optionValue) + "] ."
        }
        return



# -*- coding: utf-8 -*-


# 将当前及随后的屏幕输入存放到脚本文件中
def echo_input(cls, fileName: str, block: str):
    try:
        f = open(fileName, "w", encoding=cls.testOptions.get("RESULT_ENCODING"))
        f.write(block)
        f.close()
        return [{
            "type": "result",
            "title": None,
            "rows": None,
            "headers": None,
            "columnTypes": None,
            "status": "File [" + str(fileName) + "] generated successful."
        }]
    except IOError as ie:
        return [{
            "type": "result",
            "title": None,
            "rows": None,
            "headers": None,
            "columnTypes": None,
            "status": "File [" + str(fileName) + "] generated failed. " + str(ie)
        }]



# -*- coding: utf-8 -*-
import copy
import os
import re
from .sqlparse import SQLFormatWithPrefix
from .apiparse import APIRequestObjectFormatWithPrefix
from .commands.assertExpression import evalExpression


def sortresult(result):
    """
        数组排序

        不能用sorted函数，需要考虑None出现在列表中特定元素的问题
        排序遵循空值最大原则
    """
    for i in range(len(result) - 1, 0, -1):
        for j in range(i - 1, -1, -1):
            bNeedExchange = False
            for k in range(0, len(result[i])):
                if len(result[i]) != len(result[j]):
                    return
                if result[i][k] is None and result[j][k] is None:
                    # 两边都是空值
                    continue
                if result[i][k] is None and result[j][k] is not None:
                    # 左边空值， 右侧不空， 按照左侧大值来考虑
                    break
                if result[j][k] is None and result[i][k] is not None:
                    # 右侧空值， 左边不空， 按照右侧大值来考虑
                    bNeedExchange = True
                    break
                if not isinstance(result[i][k], type(result[j][k])):
                    if str(result[i][k]) < str(result[j][k]):
                        bNeedExchange = True
                        break
                    if str(result[i][k]) > str(result[j][k]):
                        break
                else:
                    if result[i][k] < result[j][k]:
                        bNeedExchange = True
                        break
                    if result[i][k] > result[j][k]:
                        break
            if bNeedExchange:
                result[j], result[i] = result[i], result[j]


def rewiteStatement(cls, statement: str, commandScriptFile: str):
    # 随后尝试被替换内容是否为当前应用变量，环境变量
    while True:
        # 替换脚本中的变量信息
        # 替换： 一： 系统的环境变量,即{{env}}
        # 替换： 二:  系统内嵌脚本中的变量{{eval}}
        match_obj = re.search(r"{{(.*?)}}", statement, re.IGNORECASE | re.DOTALL)
        if match_obj:
            beforeRewriteStatement = statement
            searchResult = str(match_obj.group(0))
            varName = str(match_obj.group(1)).strip()

            # 先尝试在MAPPING文件中进行查找替换
            mappingResult = cls.cmdMappingHandler.RewriteWord(commandScriptFile, varName)
            if varName != mappingResult:
                statement = statement.replace(searchResult, str(mappingResult))

            # 尝试本地变量
            try:
                evalResult = evalExpression(cls, varName)
                if varName != evalResult:
                    statement = statement.replace(searchResult, str(evalResult))
            except NameError:
                # 非环境变量
                pass
            # 尝试环境变量
            if varName in os.environ:
                statement = statement.replace(searchResult, os.environ[varName])

            if statement == beforeRewriteStatement:
                # 循环替换再也没有发生变化
                break
        else:
            # 没有任何可以替换的了
            break
    return statement


def rewriteHintStatement(cls, statement: str, commandScriptFile: str):
    # 命令可能会被多次改写
    rewrotedCommandHistory = []

    # 保留原脚本
    rawStatement = statement

    # 开始替换
    statement = rewiteStatement(cls=cls, statement=statement, commandScriptFile=commandScriptFile)

    if rawStatement != statement:
        # 记录被变量信息改写的命令
        rewrotedCommandHistory.append(statement)

    return statement, rewrotedCommandHistory


def rewriteSQLStatement(cls, statement: str, commandScriptFile: str):
    # 命令可能会被多次改写
    rewrotedCommandHistory = []

    # 保留原脚本
    rawStatement = statement

    # 开始替换
    statement = rewiteStatement(cls=cls, statement=statement, commandScriptFile=commandScriptFile)

    # 语句发生了变化
    if rawStatement != statement:
        # 记录被变量信息改写的命令
        rewrotedCommandHistory.append(
            SQLFormatWithPrefix("Your SQL has been changed to:\n" + statement, 'REWROTED '))

    return statement, rewrotedCommandHistory


def rewriteAPIStatement(cls, requestObject: [], commandScriptFile: str):
    # 命令可能会被多次改写
    rewrotedRequestObjects = []

    # 保留原脚本
    rawRequestObject = copy.copy(requestObject)

    # 替换请求目标的信息
    httpRequestTarget = rewiteStatement(
        cls=cls,
        statement=rawRequestObject["httpRequestTarget"],
        commandScriptFile=commandScriptFile)

    # 替换正文信息
    if "contents" in rawRequestObject:
        httpRequestContents = copy.copy(rawRequestObject["contents"])
        # 开始替换
        for nPos in range(0, len(httpRequestContents)):
            newHttpRequestContent = rewiteStatement(
                cls=cls,
                statement=httpRequestContents[nPos],
                commandScriptFile=commandScriptFile)
            if newHttpRequestContent == httpRequestContents[nPos]:
                # 一直循环到没有任何东西可以被替换
                break
            else:
                httpRequestContents[nPos] = newHttpRequestContent
    else:
        httpRequestContents = None

    # 替换请求字段
    if "httpFields" in rawRequestObject:
        httpRequestFields = copy.copy(rawRequestObject["httpFields"])
        for key, value in httpRequestFields.items():
            originFieldName = key
            key = rewiteStatement(
                cls=cls,
                statement=originFieldName,
                commandScriptFile=commandScriptFile)
            if originFieldName != key:
                # 替换Fields中的Field字段
                del httpRequestFields[originFieldName]
            value = rewiteStatement(
                cls=cls,
                statement=value,
                commandScriptFile=commandScriptFile)
            # 替换Fields中的Value字段
            httpRequestFields[key] = value
    else:
        httpRequestFields = None

    # 更新回原请求对象
    requestObject["httpRequestTarget"] = httpRequestTarget
    if httpRequestContents is not None:
        requestObject["contents"] = httpRequestContents
    if httpRequestFields is not None:
        requestObject["httpFields"] = httpRequestFields

    # 语句发生了变化
    if rawRequestObject != requestObject:
        # 记录被变量信息改写的命令
        rewrotedRequestObjects.append(
            APIRequestObjectFormatWithPrefix(
                headerPrefix="Your API has been changed to:\n",
                requestObject=requestObject,
                outputPrefix='REWROTED '))

    return requestObject, rewrotedRequestObjects


def parseSQLHints(commandHints: list):
    commandHintList = {}

    for commandHint in commandHints:
        # [Hint]  Scenario:XXXX   -- 相关SQL的Scenariox信息，仅仅作为日志信息供查看
        match_obj = re.match(
            r"^Scenario:(.*)", commandHint, re.IGNORECASE | re.DOTALL)
        if match_obj:
            senario = match_obj.group(1).strip()
            if len(senario.split(':')) == 2:
                # 如果包含两个内容， 规则是:Scenario:<ScenarioId>:<ScenarioName>
                scenarioSplitList = senario.split(':')
                commandHintList["ScenarioId"] = scenarioSplitList[0].strip()
                commandHintList["ScenarioName"] = scenarioSplitList[1].strip()
                continue
            else:
                # 如果只有一个内容， 规则是:Scenario:ScenarioName
                commandHintList["ScenarioId"] = "N/A"
                commandHintList["ScenarioName"] = senario
                continue

        # [Hint]  order           -- TestCli将会把随后的SQL语句进行排序输出，原程序的输出顺序被忽略
        match_obj = re.search(r"^order", commandHint, re.IGNORECASE | re.DOTALL)
        if match_obj:
            commandHintList["Order"] = True
            continue

        # [Hint]  LogFilter      -- TestCli会过滤随后显示的输出信息，对于符合过滤条件的，将会被过滤
        match_obj = re.search(
            r"^LogFilter(\s+)(.*)", commandHint, re.IGNORECASE | re.DOTALL)
        if match_obj:
            # 可能有多个Filter信息
            sqlFilter = match_obj.group(2).strip()
            if "LogFilter" in commandHintList:
                commandHintList["LogFilter"].append(sqlFilter)
            else:
                commandHintList["LogFilter"] = [sqlFilter, ]
            continue

        # [Hint]  LogMask      -- TestCli会掩码随后显示的输出信息，对于符合掩码条件的，将会被掩码
        match_obj = re.search(
            r"^LogMask(\s+)(.*)", commandHint, re.IGNORECASE | re.DOTALL)
        if match_obj:
            sqlMask = match_obj.group(2).strip()
            if "LogMask" in commandHintList:
                commandHintList["LogMask"].append(sqlMask)
            else:
                commandHintList["LogMask"] = [sqlMask]
            continue

        # [Hint]  SQL_DIRECT   -- TestCli执行的时候将不再尝试解析语句，而是直接解析执行
        match_obj = re.search(r"^SQL_DIRECT", commandHint, re.IGNORECASE | re.DOTALL)
        if match_obj:
            commandHintList["SQL_DIRECT"] = True
            continue

        # [Hint]  SQL_PREPARE   -- TestCli执行的时候将首先尝试解析语句，随后执行
        match_obj = re.search(r"^SQL_PREPARE", commandHint, re.IGNORECASE | re.DOTALL)
        if match_obj:
            commandHintList["SQL_PREPARE"] = True
            continue
    return commandHintList


def parseAPIHints(commandHints: list):
    commandHintList = {}

    for commandHint in commandHints:
        # [Hint]  Scenario:XXXX   -- 相关SQL的Scenariox信息，仅仅作为日志信息供查看
        match_obj = re.match(
            r"^Scenario:(.*)", commandHint, re.IGNORECASE | re.DOTALL)
        if match_obj:
            senario = match_obj.group(1).strip()
            if len(senario.split(':')) == 2:
                # 如果包含两个内容， 规则是:Scenario:<ScenarioId>:<ScenarioName>
                scenarioSplitList = senario.split(':')
                commandHintList["ScenarioId"] = scenarioSplitList[0].strip()
                commandHintList["ScenarioName"] = scenarioSplitList[1].strip()
                continue
            else:
                # 如果只有一个内容， 规则是:Scenario:ScenarioName
                commandHintList["ScenarioId"] = "N/A"
                commandHintList["ScenarioName"] = senario
                continue

        # [Hint]  LogFilter      -- TestCli会过滤随后显示的输出信息，对于符合过滤条件的，将会被过滤
        match_obj = re.search(
            r"^LogFilter(\s+)(.*)", commandHint, re.IGNORECASE | re.DOTALL)
        if match_obj:
            # 可能有多个Filter信息
            sqlFilter = match_obj.group(5).strip()
            if "LogFilter" in commandHintList:
                commandHintList["LogFilter"].append(sqlFilter)
            else:
                commandHintList["LogFilter"] = [sqlFilter, ]
            continue

        # [Hint]  LogMask      -- TestCli会掩码随后显示的输出信息，对于符合掩码条件的，将会被掩码
        match_obj = re.search(
            r"^LogMask(\s+)(.*)", commandHint, re.IGNORECASE | re.DOTALL)
        if match_obj:
            sqlMask = match_obj.group(5).strip()
            if "LogMask" in commandHintList:
                commandHintList["LogMask"].append(sqlMask)
            else:
                commandHintList["LogMask"] = [sqlMask]
            continue

    return commandHintList



# -*- coding: utf-8 -*-
import copy

from ..helper import helpMessage


def showHelp(cls, topicName: str):
    if topicName == "":
        headers = ["COMMAND", "SUMMARY"]
        rows = []
        for helpTopic in helpMessage:
            row = (helpTopic["topic"].upper(), helpTopic["summary"])
            rows.append(row)
        yield {
            "type": "result",
            "title": None,
            "rows": rows,
            "headers": headers,
            "columnTypes": None,
            "status": "Use \"_HELP <command>\" to get detail help messages."
        }
    else:
        helpTopic = None
        for item in helpMessage:
            if item["topic"].upper() == topicName.upper():
                helpTopic = copy.copy(item)
                break
        if helpTopic is None:
            yield {
                "type": "error",
                "message": "Unrecognized command [" + topicName.upper() + "]. " +
                           "Use \"_HELP\" to list all commands."
            }
            return
        status = "  Command:"
        status = status + "\n    " + str(topicName).upper()
        status = status + "\n"
        status = status + "\n" + "  Summary:"
        status = status + "\n" + "    " + helpTopic["summary"]
        status = status + "\n"
        status = status + "\n" + "  Synatx:"
        synatx = helpTopic["synatx"]
        synatxLines = synatx.split('\n')
        if len(synatxLines[0].strip()) == 0:
            # 去掉第一个空行
            synatxLines = synatxLines[1:]
        lspaceLen = len(synatxLines[0]) - len(synatxLines[0].lstrip())
        for nPos in range(0, len(synatxLines)):
            synatxLines[nPos] = "    " + synatxLines[nPos][lspaceLen:]
        synatx = "\n".join(synatxLines)
        status = status + "\n" + synatx
        yield {
            "type": "result",
            "title": None,
            "rows": None,
            "headers": None,
            "columnTypes": None,
            "status": status
        }
